# -*- coding: utf-8 -*-
#
# File: replicator.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

import logging
from DateTime import DateTime
from time import strptime
import csv
import cStringIO
import re
from exceptions import *

from zope.interface import implements
from zope import event
from zope.component import getUtility, getAdapters

from Products.CMFCore.utils import getToolByName
from Products.csvreplicata import config

if config.PLONE25:
    from zope.app.event.objectevent import ObjectCreatedEvent  as ObjectInitializedEvent
    from zope.app.event.objectevent import ObjectModifiedEvent as ObjectEditedEvent
else:
    # plone > 2.5
    from Products.Archetypes.event import ObjectInitializedEvent
    from Products.Archetypes.event import ObjectEditedEvent

from interfaces import Icsvreplicata
from interfaces import ICSVReplicataObjectsSearcher
from interfaces import ICSVReplicataExportPlugin
#from config import getHandlers, default_handler

from Products.csvreplicata import getPortalTypes

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFPlone.utils import getSiteEncoding


re_flags = re.M|re.S|re.U
logger = logging.getLogger('CSV REPLICATOR')

REMOVE_SPECIALCHARS = re.compile(u"((\x0d\x0a)|(\r\n))", re_flags)
DOUBLELINER = re.compile(u"\n\s*(\n\s*)+", re_flags)
EMPTYLINE  = re.compile(u"^()$", re_flags)


def fill_empty_fields_and_write_rows(writer, ids, labels=None, rows=None):
    if ids:
        writer.writerow(ids)
    if labels:
        writer.writerow(labels)
    if rows:
        # fill empty field left by non value
        # eg plugin that does not match for a particular object
        for index, row in enumerate(rows[:]):
            if ids:
                if len(row) < len(ids):
                    for i in range(len(ids) - len(row)):
                        rows[index].append(None)
            writer.writerow(rows[index])

class Replicator(object):
    """ a folder able to import/export its content as CSV
    """
    implements(Icsvreplicata)

    def __init__(self, context):
        """Initialize adapter."""
        self.context = context
        self.flag = True
        self.site_encoding = getSiteEncoding(context)

    def csvimport(self, csvfile, encoding=None, delimiter=None,
                  stringdelimiter=None, datetimeformat=None,
                  conflict_winner="SERVER", wf_transition=None, zip=None,
                  vocabularyvalue="No", count_created=0, count_modified=0,
                  errors=[]):
        """
        CSV import.

        Calls recursively self._csvimport while self.flag
        """
        while self.flag :

            count_created, count_modified, export_date, errors = \
                    self._csvimport(csvfile, encoding, delimiter,
                                    stringdelimiter, datetimeformat,
                                    conflict_winner, wf_transition, zip,
                                    vocabularyvalue, count_created,
                                    count_modified, errors)

        return (count_created, count_modified, export_date, errors)

    def _csvimport(self, csvfile, encoding, delimiter, stringdelimiter,
                   datetimeformat, conflict_winner, wf_transition, zip,
                   vocabularyvalue, count_created, count_modified, errors):

            csvfile.seek(0)

            # get portal types
            types = getPortalTypes(self.context)

            # read parameters
            csvtool = getToolByName(self.context, "portal_csvreplicatatool")
            if encoding is None:
                encoding = csvtool.getEncoding()
            if delimiter is None:
                delimiter = csvtool.getDelimiter()
            if stringdelimiter is None:
                stringdelimiter = csvtool.getStringdelimiter()
            if datetimeformat is None:
                #datetimeformat = csvtool.getDateTimeFormat()
                datetimeformat = "%Y-%m-%d"
            self.datetimeformat = datetimeformat
            self.vocabularyvalue = vocabularyvalue


            # read csv
            reader = csv.reader(csvfile, delimiter=delimiter,
                                quotechar=stringdelimiter,
                                quoting=csv.QUOTE_NONNUMERIC)
            line = 1
            # parse header
            head = reader.next()
            export_folder = head[0]
            export_date_str = str(head[1])
            export_date = DateTime(int(export_date_str[0:4]),
                                   int(export_date_str[4:6]),
                                   int(export_date_str[6:8]),
                                   int(export_date_str[8:10]),
                                   int(export_date_str[10:12]),
                                   int(export_date_str[12:14]))

            # parse content
            specific_fields = None
            label_line = False
            needs_another_loop = False
            for row in reader:
                line = line + 1
                if not label_line:
                    # read type fields
                    if row[0] == "parent":
                        specific_fields = row[3:]
                        # next line must not be considered (labels line)
                        label_line = True
                    # read values
                    else:
                        try:
                            (is_new, is_modified) = \
                                            self.importObject(row,
                                                              specific_fields,
                                                              types[row[2]],
                                                              conflict_winner,
                                                              export_date,
                                                              wf_transition,
                                                              zip,
                                                              encoding=encoding,
                                                             )
                            if is_new:
                                count_created = count_created + 1
                            elif is_modified:
                                count_modified = count_modified + 1

                        except csvreplicataConflictException, e:
                            #errors.append("Conflict on line " + \
                                          #str(line)+": %s" % (e))

                            # here content is modified during a second
                            # (or more) parsing "
                            pass

                        except csvreplicataBrokenReferenceException, e:
                            needs_another_loop = True

                        except csvreplicataNonExistentContainer, e:
                            needs_another_loop = True
                            pass

                        except csvreplicataMissingFileInArchive, e:
                            errors.append("Error in line "+str(line) + ": %s" % (e))
                            pass

                        except Exception, e:
#                            errors.append("Error in line "+str(line) + \
#                                          ": %s" % (e))
                            raise Exception, "Error in csv file line "+str(line) + ": %s" % (e)

                else:
                    label_line = False

            self.flag = needs_another_loop

            return (count_created, count_modified, export_date, errors)



    def importObject(self, row, specific_fields, type, conflict_winner,
                     export_date, wf_transition, zip, encoding='utf-8'):
        """
        """
        modified = False
        is_new_object = False
        protected = True
        parent_path = row[0]
        id = row[1]
        type_class = row[2]
        if parent_path == "":
            container = self.context
        else:
            try:
                container = self.context.unrestrictedTraverse(parent_path)
            except:
                raise csvreplicataNonExistentContainer, \
                "Non existent container %s " % parent_path


        obj = getattr(container, id, None)
        if obj is None:
            # object does not exist, let's create it
            attool = getToolByName(self.context, 'archetype_tool')
            at = attool.lookupType(type[0], type[1])
            portal_type = at['portal_type']
            container.invokeFactory(portal_type, id=id)
            obj = getattr(container, id)
            is_new_object = True
            protected = False
        else:
            # object exists, so check conflicts
            lastmodified = obj.modified()
            if lastmodified > export_date:
                if conflict_winner == "LOCAL":
                    protected = False
            else:
                protected = False

        # update object
        csvtool = getToolByName(self.context, "portal_csvreplicatatool")
        handlers = csvtool.getHandlers()
        i = 3
        for f in specific_fields:
            if f is not None and f != "":
                type = obj.Schema().getField(f).getType()
                h = handlers.get(type, handlers['default_handler'])
                v = None
                try:
                    v = row[i].decode(encoding)
                except:
                    v = row[i].encode('utf-8').decode(encoding)
                handler = h['handler_class']
                # XXX:on plone25, 'allowDiscussion' is stored in a stringfield, force to be bool.
                if f == 'allowDiscussion':
                    h = handlers.get('Products.Archetypes.Field.BooleanField')
                handler = h['handler_class']
                old_value = handler.get(obj, f, context=self)
                if isinstance(old_value, basestring):
                    if not isinstance(old_value, unicode) and isinstance(v, unicode):
                        try:
                            old_value = old_value.decode('utf-8')
                        except:
                            pass
                if old_value != v:
                    if protected:
                        raise csvreplicataConflictException, \
                        "Overlapping content modified on" \
                        " the server after exportation"
                    else:
                        modified = True
                        if h['file']:
                            handler.set(obj, f, v, context=self, zip=zip, parent_path=parent_path)
                        else:
                            handler.set(obj, f, v, context=self)

            i = i+1

        # call events
        if is_new_object:
            event.notify(ObjectInitializedEvent(obj))
            obj.unmarkCreationFlag()
            obj.at_post_create_script()
            obj.indexObject()
            try:
                wftool = getToolByName(self.context, 'portal_workflow')
                wftool.doActionFor( obj, wf_transition)
            except Exception:
               pass
        elif modified:
            event.notify(ObjectEditedEvent(obj))
            obj.at_post_edit_script()
            obj.reindexObject()

        return (is_new_object, modified)

    def csvexport(self,
                  encoding = None, delimiter = None, stringdelimiter = None,
                  datetimeformat = None,
                  depth = 1, wf_states = None,
                  zip = None,
                  vocabularyvalue = "No",
                  exportable_content_types = None):
        """
        """
        # read parameters
        csvtool = getToolByName(self.context, "portal_csvreplicatatool")
        if encoding is None:
            encoding = csvtool.getEncoding()
        if delimiter is None:
            delimiter = csvtool.getDelimiter()
        if stringdelimiter is None:
            stringdelimiter = csvtool.getStringdelimiter()
        if datetimeformat is None:
            datetimeformat = csvtool.getDateTimeFormat()
            #datetimeformat = "%Y-%m-%d"
        self.datetimeformat = datetimeformat
        self.vocabularyvalue = vocabularyvalue

        # initialize csv
        stream = cStringIO.StringIO()
        writer = csv.writer(stream, delimiter=delimiter,
                            quotechar=stringdelimiter,
                            quoting=csv.QUOTE_NONNUMERIC)

        writer.writerow(["/".join\
                         (self.context.getPhysicalPath()).encode(encoding),
                         DateTime().strftime(format='%Y%m%d%H%M%S').\
                         encode(encoding)])

        # search objects
        #exportable_content_types = csvtool.getReplicableTypesSorted()
        if exportable_content_types is not None:
            if self.context.Type() == "Smart Folder":
                all = self.context.queryCatalog(full_objects=True)
                exportable_objects = []
                for o in all:
                    if o.Type() in exportable_content_types:
                        exportable_objects.append(o)
            else:
                query = {'portal_type': exportable_content_types}
                if depth == 0:
                    path = {'query':"/".join(self.context.getPhysicalPath())}
                else:
                    path = {'query':"/".join(self.context.getPhysicalPath()),
                            'depth':depth}
                query['path'] = path

                if wf_states is not None:
                    query['review_state'] = wf_states

                search_exportable = self.context.portal_catalog.searchResults(
                    query)
                exportable_objects = [o.getObject() for o in search_exportable]
        else:
            exportable_objects = []

        # search plugins for objects to add to the export list
        objects_searchers = getAdapters([self, self.context],
                              ICSVReplicataObjectsSearcher)
        # be sure not to export twice the same content.
        objects = []
        for oid, o_s in objects_searchers:
            # we do not need to filter on object meta types as plugins can export everything :)
            # the configurable list of objects is only available for AT based contents.
            noecho = [objects.append(o)
                      for o in o_s.getObjects()
                      if (
                          (not (o in exportable_objects))
                          and (not o in objects)
                      )
                      ]
        exportable_objects.extend(objects)

        # export content
        currentrows = []
        currenttype = None
        currentfields = []
        currentpluginfields = []
        currentlabels = []
        currentfieldsIndex = 0
        for index, obj in enumerate(exportable_objects):
            # search plugins that can add cells to our objects
            plugins = getAdapters([self, obj], ICSVReplicataExportPlugin)
            type_info = str(obj.getTypeInfo().id)
            if not(type_info == currenttype):
                currenttype = type_info
                # do no write to file the fist iteration step
                # goal is to flush to file per object type to minimize
                # RAM memory usage
                if index > 0:
                    fill_empty_fields_and_write_rows(
                        writer,
                        currentfields + currentpluginfields,
                        currentlabels + currentpluginfields,
                        currentrows
                    )
                    # be sure, reset!
                    currentfields, currentlabels, currentrows = [], [], []
                # get type fields
                currentFieldsAndLabels = self.getTypeFields(type_info)
                # reset fields row
                currentfields = [s[0].encode(encoding)
                                 for s in currentFieldsAndLabels]
                # reset labels row
                for s in currentFieldsAndLabels:
                    # handle utf-8 false-ascii encoded string
                    try:
                        currentlabels.append(s[1].encode(encoding))
                    except UnicodeDecodeError:
                        currentlabels.append(s[1].decode(encoding).encode(encoding))
                # per object specific fields list
                current_specific_fields = currentfields[3:]
                # reset current plugin handled fields
                currentpluginfields = []
            # add cells given by plugins, must run once per object as each object
            # even on the same type can have fields like for annotations
            for pluginid, plugin in plugins:
                plugin.append_ids(currentpluginfields)
            # initiate to blank all valuees currently given by plugins
            plugins_values = [None for i in range(len(currentpluginfields))]
            # fetch the possible objecvalues handled by plugins
            for pluginid, plugin in plugins:
                plugin.fill_values(plugins_values, currentpluginfields)
            values_row = self.getObjectValues(obj,
                                              current_specific_fields,
                                              zip) + plugins_values
            for i, value in enumerate(values_row[:]):
                if isinstance(value, basestring):
                    # byte but not utf-9 yet
                    if not isinstance(value, unicode):
                        values_row[i] = value.decode('utf-8')
                    # any way in anycase, back to ASCII encoded
                    values_row[i] = values_row[i].encode(encoding)
            currentrows.append(values_row)
        # last loop turn fields
        if len(currentrows):
            fill_empty_fields_and_write_rows(
                writer,
                currentfields + currentpluginfields,
                currentlabels + currentpluginfields,
                currentrows
            )
        return stream.getvalue()

    def getTypeFields(self, type):
        """
        """
        fields_and_labels = [('parent', 'Parent folder'),
                             ('id'    , 'Identifier'),
                             ('type'  , 'Content type')]
        types = {}
        try:
            attool = getToolByName(self.context, 'archetype_tool')
            types = getPortalTypes(self.context)
        except:
            pass
        at = None
        # if its not an archetypes based object
        # we will rely on present plugins to export its specific fields
        try:
            at = attool.lookupType(types[type][0], types[type][1])
        except KeyError, e:
            return fields_and_labels
        # All right, archetypes, keep on workin'
        csvtool = getToolByName(self.context, "portal_csvreplicatatool")
        if csvtool.replicabletypes.has_key(type):
            schematas = csvtool.replicabletypes[type]
        else:
            schematas = []
        notExportableFieldClasses = csvtool.getExcludedfieldsclasses()
        notExportableFields = csvtool.getExcludedfields()
        at_class = at['klass']
        for schemata in schematas:
            fields = at_class.schema.getSchemataFields(schemata)
            #TODO: use getTranslationService to get the i18n translation
            fields_and_labels.extend(
                [(f.getName(), f.widget.label)
                 for f in fields
                 if f.__class__.__name__ not in notExportableFieldClasses
                 and f.getName() not in notExportableFields]
            )
        return fields_and_labels

    def getObjectValues(self, obj, specific_fields, zip):
        """
        """
        # compute parent path
        current = "/".join(self.context.getPhysicalPath())
        parent_path = "/".join(obj.getParentNode().getPhysicalPath())
        if parent_path == current:
            parent_path = ""
        elif parent_path.startswith(current+"/"):
            parent_path = parent_path[len(current)+1:]
        # add the 3 standard first columns
        obj_id, type_info = None, None
        # deal with  specific none zope object
        try:
            type_info = obj.getTypeInfo().id
        except:
            pass
        try:
            obj_id = obj.id
        except:
            pass
        values = [parent_path, obj_id, type_info]
        csvtool = getToolByName(self.context, "portal_csvreplicatatool")
        handlers = csvtool.getHandlers()
        # add specific columns
        for f in specific_fields:
            type = obj.Schema().getField(f).getType()

            h = handlers.get(type, handlers['default_handler'])
            # XXX:on plone25, 'allowDiscussion' is stored in a stringfield, force to be bool.
            if f == 'allowDiscussion':
                h = handlers.get('Products.Archetypes.Field.BooleanField')
            handler = h['handler_class']
            if h['file']:
                v = handler.get(obj, f, context=self, zip=zip, parent_path=parent_path)
            else:
                v = handler.get(obj, f, context=self)
            values.append(v)
        return values





from handlers import base
from handlers import reference
from handlers import file

import logging
logger = logging.getLogger('CONFIG')

# settings default archetypes widgets fields
HANDLERS={'default_handler': {'handler_class' : base.CSVdefault(), 'file' : False},
          'Products.Archetypes.Field.StringField': {'handler_class' : base.CSVString(), 'file' : False},
          'Products.Archetypes.Field.IntegerField': {'handler_class' : base.CSVInteger(), 'file' : False},
          'Products.Archetypes.Field.FloatField': {'handler_class' : base.CSVFloat(), 'file' : False},
          'Products.Archetypes.Field.BooleanField': {'handler_class' : base.CSVBoolean(), 'file' : False},
          'Products.Archetypes.Field.LinesField': {'handler_class' : base.CSVLines(), 'file' : False},
          'Products.Archetypes.Field.TextField': {'handler_class' : base.CSVText(), 'file' : False},
          'Products.Archetypes.Field.DateTimeField': {'handler_class' : base.CSVDateTime(), 'file' : False},
          'Products.Archetypes.Field.ReferenceField': {'handler_class' : reference.CSVReference(), 'file' : False},
          'Products.Archetypes.Field.FileField': {'handler_class' : file.CSVFile(), 'file' : True},
          'Products.Archetypes.Field.ImageField': {'handler_class' : file.CSVFile(), 'file' : True},
          'plone.app.blob.subtypes.file.ExtensionBlobField': {'handler_class' : file.CSVFile(), 'file' : True},
          'Products.AttachmentField.AttachmentField.AttachmentField': {'handler_class' : file.CSVFile(), 'file' : True},
          }




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2009, Mathieu PASQUET <mpa@makina-corpus.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the <ORGANIZATION> nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

__docformat__ = 'restructuredtext en'


class CSVReplicataPluginAbstract(object):
    """."""

    def __init__(self, replicator, context):
        self.replicator = replicator
        self.context = context

class CSVReplicataObjectSearcherAbstract(CSVReplicataPluginAbstract):
    """Just to be marked in zcml."""

    def getObjects(self):
        """."""
        raise Exception('Not implemented.')

class CSVReplicataExportPluginAbstract(CSVReplicataPluginAbstract):
    """."""

    def __init__(self, replicator, context):
        CSVReplicataPluginAbstract.__init__(self, replicator, context)
        self.ids = []
        self._datetimeformat = None

    def append_ids(self, row_ids):
        """."""
        row_ids.extend(
            [i 
             for i in self.ids 
             if not i in row_ids]
        )

    def fill_values(self, row, row_ids):
        """."""
        raise Exception('Not implemented.')

    def set_values(self, row, row_ids):
        """."""
        raise Exception('Not implemented.')

# example plugins for searching/exporting comments on plone2.5

# THE ZCML REGISTRATION WILL LOOK LIKE:
"""
  <adapter
      name="comments_searcher"
      factory=".adapters.CommentsObjectsSearcher"
      provides="Products.csvreplicata.interfaces.ICSVReplicataObjectsSearcher"
      for="Products.csvreplicata.interfaces.Icsvreplicata Products.CMFCore.interfaces._tools.IDiscussionTool"
    />    
  <adapter
      name="comments_exporter"
      factory=".adapters.CommentsExporter"
      provides="Products.csvreplicata.interfaces.ICSVReplicataExportPlugin"
      for="Products.csvreplicata.interfaces.Icsvreplicata zope.interface.Interface"
    /> 

    After, mark your portal_discussion with "ICSVReplicable" and export it.
"""


# THE CODE WILL LOOK LIKE

#import logging
#from datetime import datetime
#from DateTime import DateTime
#
#from zope.app.annotation.interfaces import IAnnotations
#from zope.component import getMultiAdapter
#from persistent.dict import PersistentDict
#from persistent.list import PersistentList
#
#from Products.CMFCore.utils import getToolByName
#from Products.csvreplicata.adapters import CSVReplicataObjectSearcherAbstract
#from Products.csvreplicata.adapters import CSVReplicataExportPluginAbstract
#from Products.csvreplicata.adapters import CSVReplicataPluginAbstract 

#def get_dict_csv_mapping(dictionnary,
#                         results = None,
#                         export_key = 'myexportkey',
#                         replicator = None
#                        ):
#    if isinstance(dictionnary, PersistentDict):
#        dictionnary = dict(dictionnary)
#    if not results:
#        results = {}
#    for key in dictionnary:
#        if isinstance(dictionnary[key], dict) or isinstance(dictionnary[key], PersistentDict):
#            get_dict_csv_mapping(dict(dictionnary[key]), results, '%s_%s' % (export_key, key), replicator)
#        else:
#            value = dictionnary[key]
#            #print '-->%s  %s <-> (%s, %s)' % (dict(dictionnary), key, value.__class__, value)
#            if (
#                isinstance(value, list) 
#                or isinstance(value, PersistentList)
#                or isinstance(value, tuple)
#            ):
#                value = ','.join(['%s' % a for a in value])
#            if isinstance(value, DateTime) or isinstance(value, datetime):
#                df = '%d/%m/%Y %H:%M:%S'
#                if replicator:
#                    getattr(replicator, 'datetimeformat', df)
#                value = value.strftime(df)
#            results['%s_%s' % (export_key, key)] = value
#    return results
#
#class CommentsObjectsSearcher(CSVReplicataObjectSearcherAbstract):
#
#    def getObjects(self):
#        logger = logging.getLogger('Products.csvreplicata.adapters.CommentsObjectsSearcher')
#        objs = []
#        c = getToolByName(self.context, 'portal_catalog')
#        bcomments = c.searchResults(**{
#            'meta_type': ['Discussion Item'],
#            'sort_on': 'in_reply_to',
#        })
#        lbcomments = len(bcomments)
#        logger.info('%s comments to export' % lbcomments)
#        SLICE = 1000
#        for i in range((lbcomments / SLICE) + 1):
#            lowerBound = SLICE * i
#            upperBound = SLICE * (i+1)
#            if upperBound > lbcomments:
#                upperBound = lbcomments
#            logger.info('Loading %s to %s comments.' % (lowerBound, upperBound))
#            #lowerBound, upperBound = 0, 3
#            scomments = [b.getObject() 
#                         for b in bcomments[lowerBound : upperBound]]
#            objs.extend(scomments)
#        logger.info('All comments loaded.')
#        return objs
#
#class CommentsExporter(CSVReplicataExportPluginAbstract):
#    """."""
#    def __init__(self, *args, **kwargs):
#        CSVReplicataExportPluginAbstract.__init__(self, *args, **kwargs)
#        self.comment = None
#        self.comment_as_dict = None
#        if self.context.meta_type in ['Discussion Item']:
#            try:
#                self.comment = {}
#                self.comment['creators']          = self.context.listCreators()
#                self.comment['contributors']      = self.context.contributors
#                self.comment['description']       = self.context.description
#                self.comment['effective_date']    = self.context.effective_date
#                self.comment['expiration_date']   = self.context.expiration_date
#                self.comment['id']                = self.context.id
#                self.comment['in_reply_to']       = self.context.in_reply_to
#                self.comment['modification_date'] = self.context.modification_date
#                self.comment['subject']           = self.context.subject
#                self.comment['text']              = self.context.text
#                self.comment['text_format']       = self.context.text_format
#                self.comment['title']             = self.context.title
#                self.comment['cooked_text']       = self.context.cooked_text
#                self.comment['language']          = self.context.language
#                self.comment['path']              = '/'.join(self.context.getPhysicalPath())
#                self.comment['in_reply_to']       = '/'.join(self.context.inReplyTo().getPhysicalPath())
#                self.comment['in_reply_to_chain'] = ['/'.join(obj.getPhysicalPath()) 
#                                                     for obj in self.context.parentsInThread()]
#                 
#                self.comment_as_dict = get_dict_csv_mapping(self.comment, 
#                                                            export_key = 'comment',
#                                                            replicator=self.replicator)
#                ks = self.comment_as_dict.keys()
#                ks.sort()
#                self.ids = ks
#            except:
#                pass
#
#    def fill_values(self, row, row_ids):
#        """."""
#        for id in row_ids:
#            if id in self.ids:
#                index = row_ids.index(id)
#                if index < len(row):
#                    row[index] = self.comment_as_dict[id]
#
#    def set_values(self, row, row_ids):
#        """."""
#        pass 
                 

# vim:set et sts=4 ts=4 tw=80:



# -*- coding: utf-8 -*-
#
# File: csvreplicata.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'


# There are three ways to inject custom code here:
#
#   - To set global configuration variables, create a file AppConfig.py.
#       This will be imported in config.py, which in turn is imported in
#       each generated class and in this file.
#   - To perform custom initialisation after types have been registered,
#       use the protected code section at the bottom of initialize().

import logging
logger = logging.getLogger('csvreplicata')
logger.debug('Installing Product')

import os
import os.path
from Globals import package_home
import Products.CMFPlone.interfaces
from Products.Archetypes import listTypes
from Products.Archetypes.atapi import *
from Products.Archetypes.utils import capitalize
from Products.CMFCore import DirectoryView
from Products.CMFCore import permissions as cmfpermissions
from Products.CMFCore import utils as cmfutils
from Products.CMFPlone.utils import ToolInit
from config import *

DirectoryView.registerDirectory('skins', product_globals)


##code-section custom-init-head #fill in your manual code here
from Products.CMFCore.utils import getToolByName

def getPortalTypes(context):
    attool = getToolByName(context, 'archetype_tool')
    types = {}
    for t in attool.listRegisteredTypes():
        types[t['portal_type']]=(t['package'], t['meta_type'])
    return types
##/code-section custom-init-head


def initialize(context):
    """initialize product (called by zope)"""
    ##code-section custom-init-top #fill in your manual code here
    ##/code-section custom-init-top

    # imports packages and types for registration

    import csvreplicataTool

    # Initialize portal tools
    tools = [csvreplicataTool.csvreplicataTool]
    ToolInit( PROJECTNAME +' Tools',
                tools = tools,
                icon='tool.gif'
                ).initialize( context )

    # Initialize portal content
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    cmfutils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = DEFAULT_ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)

    ##code-section custom-init-bottom #fill in your manual code here
    ##/code-section custom-init-bottom




#
from Products.csvreplicata.interfaces import ICSVReplicable
from Products.csvreplicata import config
from Products.Five import BrowserView

class isCSVReplicable(object):

    def __call__(self):
        return ICSVReplicable.providedBy(self.context)

class isCSVRPlone25(BrowserView):

    def __call__(self):
        return config.PLONE25

class p25wrapper_is_locked_for_current_user(BrowserView):
    """."""

    def __call__(self, *args, **kwargs):
        """."""
        if config.PLONE25:
            return False
        else:
            return self.context.restrictedTraverse(
                '@@plone_lock_info'
            ).is_locked_for_current_user() 



# -*- coding: utf-8 -*-
#
# File: csvreplicataTool.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from ZODB.PersistentMapping import PersistentMapping
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.csvreplicata.config import *


from Products.CMFCore.utils import UniqueObject


##code-section module-header #fill in your manual code here
from Products.CMFCore.utils import getToolByName
import logging
logger = logging.getLogger('csvreplicataTool')
logger.info('csvreplicataTool')

from Products.csvreplicata import getPortalTypes

##/code-section module-header

schema = Schema(
    (
    StringField(
        name='encoding',
        default="UTF-8",
        widget=StringField._properties['widget'](
            label='Encoding',
            label_msgid='csvreplicata_label_encoding',
            i18n_domain='csvreplicata',
        ),
    ),
    StringField(
        name='delimiter',
        default=";",
        widget=StringField._properties['widget'](
            label='Delimiter',
            label_msgid='csvreplicata_label_delimiter',
            i18n_domain='csvreplicata',
        ),
    ),
    StringField(
        name='stringdelimiter',
        default='"',
        widget=StringField._properties['widget'](
            label='Stringdelimiter',
            label_msgid='csvreplicata_label_stringdelimiter',
            i18n_domain='csvreplicata',
        ),
    ),
    StringField(
        name='serverfilepath',
        default="",
        widget=StringField._properties['widget'](
            label='Server import folder',
            label_msgid='csvreplicata_label_serverfilepath',
            i18n_domain='csvreplicata',
        ),
    ),
    LinesField(
        name='excludedfields',
        default=['id', 'locallyAllowedTypes', 'constrainTypesMode',
                 'immediatelyAddableTypes'],
        widget=LinesField._properties['widget'](
            label='Excludedfields',
            label_msgid='csvreplicata_label_excludedfields',
            i18n_domain='csvreplicata',
        ),
    ),
    LinesField(
        name='excludedfieldsclasses',
        widget=LinesField._properties['widget'](
            label='Excludedfieldsclasses',
            label_msgid='csvreplicata_label_excludedfieldsclasses',
            i18n_domain='csvreplicata',
        ),
    ),
    StringField(
        name='DateTimeFormat',
        default='%d/%m/%Y %H:%M:%S',
        widget=StringField._properties['widget'](
                label='Excludedfieldsclasses',
                label_msgid='csvreplicata_label_DateTimeFormat',
                i18n_domain='csvreplicata',
                ),
    ),
    ),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

csvreplicataTool_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class csvreplicataTool(UniqueObject, BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()
    implements(interfaces.IcsvreplicataTool)

    meta_type = 'csvreplicataTool'
    _at_rename_after_creation = True

    schema = csvreplicataTool_schema

    handlers = PersistentMapping

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header




    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_csvreplicatatool')
        self.setTitle('')

        ##code-section constructor-footer #fill in your manual code here
        self.setTitle('CSV Replicator tool')
        self.replicabletypes = {}
        self.handlers = {}
        ##/code-section constructor-footer

    def manage_afterAdd(self, item, container):
        """ initialize handlers with appconfig HANDLERS values"""
        self.handlers = HANDLERS

    # tool should not appear in portal_catalog
    def at_post_edit_script(self):
        self.unindexObject()

        ##code-section post-edit-method-footer #fill in your manual code here
        ##/code-section post-edit-method-footer


    # Methods

    # Manually created methods

    def setCSVsettings(self, REQUEST):
        """
        """
        self.setEncoding(REQUEST.get('encoding'));
        self.setDelimiter(REQUEST.get('delimiter'));
        self.setServerfilepath(REQUEST.get('serverfilepath'));
        self.setDateTimeFormat(REQUEST.get('datetimeformat'));

        # Redirection of the page now that the treatment is done
        REQUEST.RESPONSE.redirect(self.absolute_url()+'/csv_settings')

    def setCSVHandledTypes(self, REQUEST):
        """
        """
        types_tool = getToolByName(self, 'archetype_tool')

        # Get of the various replicabletypes current and new)
        newreplicabletypes = REQUEST.get('csvhandledtypes')
        if (not type(newreplicabletypes) is list):
            newreplicabletypes = [newreplicabletypes]
        currentreplicablestypes = self.replicabletypes

        # Addition of the new replicable types, by default a new empty list
        # and creation of a temp variable to hold the new csv handled types
        newreplicabletypestempdict = {}
        for t in newreplicabletypes :
            newreplicabletypestempdict[t] = []
            if (not currentreplicablestypes.has_key(t)):
                    currentreplicablestypes[t] = ['default', ]

        # removal of the types that are not anymore replicables
        for k, v in currentreplicablestypes.items():
            if (not newreplicabletypestempdict.has_key(k)):
                del currentreplicablestypes[k]

        # save of the new values
        self.replicabletypes = currentreplicablestypes

        # Redirection of the page now that the treatment is done
        REQUEST.RESPONSE.redirect(self.absolute_url()+'/csv_settings')

    def setExcludedFields(self, REQUEST):
        """
        """
        self.setExcludedfieldsclasses(
            REQUEST.get('excludedfieldsclasses').split('\n'));
        self.setExcludedfields(REQUEST.get('excludedfields').split('\n'));

        # Redirection of the page now that the treatment is done
        REQUEST.RESPONSE.redirect(self.absolute_url()+'/csv_settings')

    def getPortalTypeNames(self):
        """
        """
        l = getPortalTypes(self).keys()
        l.sort()
        return [(k, k) for k in l]

    def getReplicableTypesSorted(self):
        """
        """
        l = self.replicabletypes.keys()
        l.sort()
        return l

    def getTypeSchematas(self, type):
        """
        """
        attool = getToolByName(self, 'archetype_tool')
        pt = getPortalTypes(self)
        if pt.has_key(type):
            (package, name) = pt[type]
            t = attool.lookupType(package, name)
            return  t['klass'].schema.getSchemataNames()
        else:
            return []

    def setCSVHandledTypesSchematas(self, REQUEST):
        """
        """
        currentreplicablestypes = self.replicabletypes

        i = 0
        for ptn in self.getReplicableTypesSorted():
            i += 1
            if (type(ptn) is tuple):
                t = ptn[0]
            else:
                t = ptn

            r = REQUEST.get('csvhandledschematas-'+str(i))
            if r:
                if (not type(r) is list):
                    r = [r]
                currentreplicablestypes[t] = r
            else :
                if currentreplicablestypes.has_key(t):
                    del currentreplicablestypes[t]

        self.replicabletypes = currentreplicablestypes

        # Redirection of the page now that the treatment is done
        REQUEST.RESPONSE.redirect(self.absolute_url()+'/csv_settings')

    def clearReplicableTypes(self):
        """
        """
        self.replicabletypes = {}

    def printReplicableTypes(self):
        """
        """
        return self.replicabletypes

    def getHandlers(self):
        """
        """
        return self.handlers

    def setHandler(self, key, value):
        """
        """
        self.handlers[key] = value

    def delHandler(self, key):
        """
        """
        del(self.handlers[key])

registerType(csvreplicataTool, PROJECTNAME)
# end of class csvreplicataTool

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# -*- coding: utf-8 -*-
#
# File: base.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from zope import interface
from Products.csvreplicata.interfaces import ICSVDefault

from DateTime import DateTime
from time import strptime

from Products.CMFCore.utils import getToolByName

from Products.csvreplicata.exceptions import *

import logging
logger = logging.getLogger('HANDLER')

class CSVdefault(object):
    """
    """
    interface.implements(ICSVDefault)
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).getRaw(obj)
        if v is None:
            return ''
        else:
            return str(v)
    
    def set(self, obj, field, value, context=None):
        if value == '':
            value = None
        self.store(field, obj, value)
        
    def store(self, field, obj, value):
        old_value = obj.Schema().getField(field).get(obj)
        if type(old_value) is tuple and type(value) is list:
            value = tuple(value)
        if obj.Schema().getField(field).writeable(obj):
            obj.Schema().getField(field).set(obj, value)
        else:
            raise csvreplicataPermissionException, \
            "Insufficient privileges to modify this object and/or field"
        
class CSVString(CSVdefault):
    """
    """
    # special case for bizarious collage vocabs.
    collage_vocab = 'Products.Collage.content._portlet.PortletVocabulary' 

    def getLenVocab(self, vocab):
        """
        Some vocabularies like the old Products.Collage.content._portlet.PortletVocabulary
        do not have __len__. So just bypass the check
        """
        lenvocab = 0
        try:
            lenvocab = len(vocab)
        except AttributeError, e:
            lenvocab = 1
        return lenvocab

    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).getRaw(obj)
        
        vocab = obj.Schema().getField(field).vocabulary


        if context.vocabularyvalue == "Yes" and self.getLenVocab(vocab):
            if hasattr(vocab, 'getValue'):
                v = vocab.getValue(v)
            # some thirdparty modules have list instead of nice vocabs.
            elif isinstance(vocab, list):
                if v in vocab:
                    return v
            elif '%s'%vocab.__class__ == self.collage_vocab:
                if v in vocab.getDisplayList(obj):
                    return vocab.getDisplayList(obj).getValue(v)
            elif hasattr(obj, vocab):
                v = getattr(obj, vocab)().getValue(v)
        if v is None:
            v = ''
        return v
    
    def set(self, obj, field, value, context=None):
        v = obj.Schema().getField(field).getRaw(obj)
        
        vocab = obj.Schema().getField(field).vocabulary
        
        lenvocab = 1
        if context.vocabularyvalue == "Yes" and self.getLenVocab(vocab):
            if isinstance(vocab, list):
                if value in vocab:
                    value = value
                else:
                    value = ''
            elif vocab.getKey(value) is not None:
                value = vocab.getKey(value)
            else:
                value = ''
        self.store(field, obj, value)
        
class CSVInteger(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj)
        if v is None:
            return ''
        else:
            return str(v)
    
    def set(self, obj, field, value, context=None):
        if value=='':
            value = None
        elif value is not None:
            value = int(value)
        self.store(field, obj, value)

class CSVFloat(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj)
        if v is None:
            return ''
        else:
            return str(v)
    
    def set(self, obj, field, value, context=None):
        if value=='':
            value = None
        elif value is not None:
            value = float(value)
        self.store(field, obj, value)

class CSVBoolean(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj)
        if v is None:
            return ''
        else:
            return str(v)
    
    def set(self, obj, field, value, context=None):
        if value=='':
            value = None
        elif value is not None:
            if value=="True":
                value=True
            elif value=="False":
                value=False
            else:
                raise csvreplicataException, field+" must be True or False"
        self.store(field, obj, value)
        
class CSVLines(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj)
        if v is None:
            return ''
        else:
            return '\n'.join(v)
    
    def set(self, obj, field, value, context=None):
        if value=='':
            value = []
        else:
            value = value.split('\n')
        self.store(field, obj, value)
        
class CSVText(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj, mimetype="text/plain")
        return v
    
    def set(self, obj, field, value, context=None):
        self.store(field, obj, value)
        
class CSVDateTime(CSVdefault):
    """
    """
    def get(self, obj, field, context=None):
        """
        """
        if context is None:
            csvtool = getToolByName(obj, "portal_csvreplicatatool")
            format = csvtool.getDateTimeFormat()
            #format = '%d/%m/%Y'
        else:
            format = context.datetimeformat
            
        v = obj.Schema().getField(field).get(obj)
        if isinstance(v, DateTime):
            return v.strftime(format)
        else:
            return ''
    
    def set(self, obj, field, value, context=None):
        if context is None:
            csvtool = getToolByName(obj, "portal_csvreplicatatool")
            format = csvtool.getDateTimeFormat()
            #format = '%d/%m/%Y'
        else:
            format = context.datetimeformat
            
        if value == '':
            value = None
        else:
            try:
                dt = strptime(value, format)
                value = DateTime(dt[0], dt[1], dt[2], dt[3], dt[4], dt[5])
                self.store(field, obj, value)
            except DateTime.DateTimeError, e:
                raise csvreplicataException, v + " is not a valid date/time"



# -*- coding: utf-8 -*-
#
# File: reference.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from Products.csvreplicata.exceptions import *
from base import CSVdefault

import logging
logger = logging.getLogger('HANDLER')
 
        
class CSVReference(CSVdefault):
    """
    """
    
    def get(self, obj, field, context=None):
        """
        """
        v = obj.Schema().getField(field).get(obj, aslist=True)
        if v is None:
            return ''
        else:
            current = "/".join(obj.getPhysicalPath())+"/"
            l = []
            for o in v:
                path = "/".join(o.getPhysicalPath())
                if path.startswith(current):
                    l.append(path[len(current):])
                else:
                    l.append(path)
            return '\n'.join(l)
    
    def set(self, obj, field, value, context=None):
        if value == '':
            ref = []
        else:
            value = value.split('\n')
            ref = []
            for path in value:
                try:
                    target = obj.unrestrictedTraverse(path)
                    ref.append(target)
                except Exception:
                    raise csvreplicataBrokenReferenceException, path + \
                    " cannot be found"
                
        self.store(field, obj, ref)
        


# -*- coding: utf-8 -*-
#
# File: file.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'


import os
import logging

from zope import interface

from Products.csvreplicata.exceptions import *
from Products.csvreplicata.interfaces import ICSVFile

from base import CSVdefault

logger = logging.getLogger('csvreplicata.HANDLER')

class CSVFile(CSVdefault):
    """
    """
    interface.implements(ICSVFile)

    def get(self, obj, field, context=None, zip=None, parent_path=''):
        """
        """
        f = obj.Schema().getField(field).get(obj)
        if not f :
            return ''
        else:
            filename = f.filename
            if zip is not None:
                #logger.error(obj.Schema().getField(field).getType())
                if obj.Schema().getField(field).getType() in \
                ("plone.app.blob.subtypes.file.ExtensionBlobField",
                 "Products.Archetypes.Field.FileField",
                 'Products.AttachmentField.AttachmentField.AttachmentField',
                 "Products.Archetypes.Field.ImageField"):

                    f = str(f.data)
                zip.writestr(os.path.join(parent_path, filename), f)
            return filename

    def set(self, obj, field, value, context=None, zip=None, parent_path = ''):
        if value == '':
            raise csvreplicataException, "No filename for %s" % (field)
        elif zip is None:
            raise csvreplicataException, "No zip file provided"
        else:
            if obj.Schema().getField(field).writeable(obj):
                try:
                    file = zip.read(os.path.join(parent_path, value))
                except KeyError, e:
                    raise csvreplicataMissingFileInArchive, "%s not found in zip file" % (value)
                obj.Schema().getField(field).set(obj, file)
                obj.Schema().getField(field).get(obj).setFilename(value)
            else:
                raise csvreplicataPermissionException, \
                "Insufficient privileges to modify this object and/or field"




# -*- coding: utf-8 -*-
#
# File: csvreplicata.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'


# Product configuration.
#
# The contents of this module will be imported into __init__.py, the
# workflow configuration and every content type module.
#
# If you wish to perform custom configuration, you may put a file
# AppConfig.py in your product's root directory. The items in there
# will be included (by importing) in this file if found.

from Products.CMFCore.permissions import setDefaultRoles
##code-section config-head #fill in your manual code here
##/code-section config-head


PROJECTNAME = "csvreplicata"

# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner'))

product_globals = globals()

# Dependencies of Products to be installed by quick-installer
# override in custom configuration
DEPENDENCIES = []

# Dependend products - not quick-installed - used in testcase
# override in custom configuration
PRODUCT_DEPENDENCIES = []

##code-section config-bottom #fill in your manual code here
##/code-section config-bottom


# Load custom configuration not managed by archgenxml
try:
    from Products.csvreplicata.AppConfig import *
except ImportError:
    pass

# A flag to support Plone 2.5.x
from Products.CMFPlone.utils import getFSVersionTuple
_ploneVersion = getFSVersionTuple()
_major = _ploneVersion[0]
_minor = _ploneVersion[1]
if (_major == 2) and (_minor <= 5):
    PLONE25 = True
else:
    PLONE25 = False



class csvreplicataException(Exception):
    pass

class csvreplicataBrokenReferenceException(Exception):
    pass

class csvreplicataConflictException(Exception):
    pass

class csvreplicataPermissionException(Exception):
    pass

class csvreplicataNonExistentContainer(Exception):
    pass

class csvreplicataMissingFileInArchive(Exception):
    pass


from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName

def removeConfiglets(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        out.write('Removing configlet %s\n' % 'CSV_replicata')
        configTool.unregisterConfiglet('CSV_replicata')

def uninstall(self):
    out = StringIO()
    removeConfiglets(self,out)



# -*- coding: utf-8 -*-

from zope.interface import Interface, Attribute

class ICSVDefault(Interface):
    """CSV Default.

    A CSV Field defaut Interface
    """

    def get(obj, field, context=None):
        """A getter."""

    def set(obj, field, value, context=None):
        """ A setter."""

    def store(field, obj, value):
        """Store field value."""


class ICSVFile(Interface):
    """CSV File.

    A CSV File defaut Interface
    """

    def get(obj, field, context=None, zip=None):
        """A getter."""

    def set(obj, field, value, context=None, zip=None):
        """ A setter."""


class IcsvreplicataTool(Interface):
    """Marker interface for .csvreplicataTool.csvreplicataTool
    """


class ICSVReplicable(Interface):
    """Marker interface for replicable folders
    """

class Icsvreplicata(Interface):
    """ adapter interface
    """

class  ICSVReplicataObjectsSearcher(Interface):
    """You have some others infos in adapters.py. """

    """Search for aditionnal objects 
    to export which are not catalogued in
    the main catalog"""

    def getObjects():
        """ Get a list of objects to export"""

class  ICSVReplicataExportPlugin(Interface):
    """You have some others infos in adapters.py"""

    """Export aditionnal fields/values pairs  
    which are not handled via archetypes handlers"""

    """Be sure to prefix your titles and not have conflicting titles 
    (Otherwise normally, first win)"""

    """ PLEASE INIT THIS IN THE __init__ OF YOUR PLUGIN """
    ids = Attribute('ordonned list of csv ids')

    def append_ids(row_ids):
        """Add the plugins ids to a list of ids tuple."""

    def fill_values(row, row_ids):
        """
        Fill contextual value in a row matching the given fields
        row: a row (list of cells) 
        As list arguments are references in python, the plugin will just
        need to  modiffy it and the caller will get the list updated as well.
        No need to return it.
        row_ids: The current list of fields matching the row.
        For example:
            a.csv has rows with a,b,c,d
            c and d are columns given by MySuperPlugin
        An appropriate call can call will be:
            current_row = [1, 2, None, None]
            MySuperPluginInstance.fill_values(
                    current_row, ['a', 'b', 'c', 'd']
            )

        You will note that the row has been initialized with empty values
        for the plugin handled columns. Indeed, the plugin is not supposed
        to handle values initialization, it just fill/update them.
        """

    def set_values(row, row_ids):
        """Set contextual value in a row matching the previous ids order.."""




from setuptools import setup, find_packages
import os
import sys

version = '1.1'

setupdir = os.path.abspath(
    os.path.dirname(__file__)
)

def read(*rnames):
    return open(
        os.path.join(setupdir, *rnames)
    ).read()

long_description = '%s' % (
    read('README.txt')          + '\n' + 
    read('docs', 'INSTALL.txt') + '\n' +
    read('docs', 'HISTORY.txt') + '\n' + 
    ''
)

if 'RST_TEST' in os.environ:
    print long_description
    sys.exit(1)

setup(name='Products.csvreplicata',
      version=version,
      description="CSV import/export for Archetypes or other contents (via plugins for the later).",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='archetypes, import/export, csv, replication, synchronisation',
      author='Eric BREHAULT',
      author_email='eric.brehault@makina-corpus.org',
      url='https://svn.plone.org/svn/collective/Products.csvreplicata',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
#
# File: browser.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from zipfile import ZipFile
from cStringIO import StringIO
import os

from Products.csvreplicata.interfaces import Icsvreplicata
from Products.csvreplicata import config

import logging
logger = logging.getLogger('CSV MANAGER')

class ReplicationManager(BrowserView):
    """ allow to manage CSV replication
    """

    def __init__(self, context, request):
        """Initialize browserview"""
        self.context = context
        self.request = request

 
    def kssClassesView(self):
        """
        Yet another plone25 wrapper.
        """ 
        if not config.PLONE25:
            return self.context.restrictedTraverse(
                '@@kss_field_decorator_view'
            )

    def getKssClasses(self):
        """
        Yet another plone25 wrapper.
        """
        try:
            kssClassesView = self.context.restrictedTraverse('@@kss_field_decorator_view')
            return kssClassesView.getKssClassesInlineEditable
        except Exception, e:
            if not config.PLONE25:
                raise

    def doImport(self):
        """
        """
        fileorigin = self.request.get("fileorigin")
        if fileorigin=="SERVER":
            csvtool = getToolByName(self.context, "portal_csvreplicatatool")
            import_dir_path = csvtool.getServerfilepath()
            file = open(os.path.join(import_dir_path, self.request.get("serverfile")))
        else:
            file = self.request.get("csvfile")

        replicator = Icsvreplicata(self.context)
        datetimeformat = self.request.get("datetimeformat")
        vocabularyvalue = self.request.get("vocabularyvalue")
        encoding = self.request.get("encoding")
        delimiter = self.request.get("delimiter")
        stringdelimiter = self.request.get("stringdelimiter")
        conflict_winner = self.request.get("conflictrule")
        wf_transition = self.request.get("wf_transition")
        if wf_transition=="None":
            wf_transition = None
        importfiles = self.request.get("importfiles")
        if importfiles=="Yes":
            zip = ZipFile(file)
            csvfile=StringIO()
            csvfile.write(zip.read("export.csv"))
            csvfile.seek(0)
        else:
            zip = None
            csvfile = file

        (count_created, 
         count_modified, 
         export_date, 
         errors) = replicator.csvimport(
             csvfile, 
             encoding = encoding, 
             delimiter = delimiter, 
             stringdelimiter = stringdelimiter, 
             datetimeformat = datetimeformat, 
             conflict_winner = conflict_winner, 
             wf_transition = wf_transition, 
             zip = zip, 
             vocabularyvalue = vocabularyvalue)
        if len(errors)==0:
            self.writeMessageOnPage(["All lines imported, %d object(s) created, %d object(s) modified." % (count_created, count_modified)])
            self.writeMessageOnPage(["This CSV file has been produced on "+str(export_date)+". To avoid inconsistencies, do not import this file anymore. It is preferable to export a new one."])
        else:
            self.writeMessageOnPage(["%d object(s) created, %d object(s) modified." % (count_created, count_modified)])
            self.writeMessageOnPage(errors, error = True)

        self.request.RESPONSE.redirect(self.context.absolute_url()+'/@@csvreplicata')

    def doExport(self):
        """
        """
        replicator = Icsvreplicata(self.context)
        encoding = self.request.get('encoding')
        depth = int(self.request.get('depth','1'))
        datetimeformat = self.request.get("datetimeformat")
        vocabularyvalue = self.request.get("vocabularyvalue")
        delimiter = self.request.get("delimiter")
        stringdelimiter = self.request.get("stringdelimiter")
        wf_states = self.request.get("wf_states", 'Any')
        selected_exportable_content_types = self.request.get("exportable_content_types")

        # handling of selected types
        exportable_content_types = selected_exportable_content_types

        if 'Any' in wf_states:
            wf_states = None
        exportfiles = self.request.get("exportfiles")

        if exportfiles=="Yes":
            # initialize zipfile
            sIO = StringIO()
            zip = ZipFile(sIO,"w",8)
        else:
            zip = None

        csv = replicator.csvexport(
            encoding = encoding, 
            delimiter = delimiter,
            stringdelimiter = stringdelimiter, 
            depth = depth, 
            datetimeformat = datetimeformat, 
            wf_states = wf_states, 
            zip = zip, 
            vocabularyvalue = vocabularyvalue, 
            exportable_content_types = exportable_content_types
        )

        if exportfiles=="Yes":
            self.request.response.setHeader('Content-type','application/zip')
            self.request.response.setHeader('Content-Disposition', "attachment; filename=export.zip")

            zip.writestr("export.csv",csv)
            zip.close()
            return sIO.getvalue()
        else:
            if not encoding:
                encoding = 'UTF-8'
            self.request.response.setHeader('Content-type','text/csv;charset='+encoding)
            self.request.response.setHeader('Content-Disposition', "attachment; filename=export.csv")
            return csv


    def writeMessageOnPage(self, messages, ifMsgEmpty = '', error = False):
        """adds portal message
        """
        plone_tools = getToolByName(self, 'plone_utils')
        if plone_tools is not None:
            #portal message type
            if error:
                msgType = 'error'
            else:
                msgType = 'info'

            #if empty
            if (len(messages)==0):
               messages = [ifMsgEmpty]

            #display it
            for msg in messages:
                if len(msg)==0:
                    mess = ifMsgEmpty
                else:
                    mess = msg
                if len(mess)>0:
                    plone_tools.addPortalMessage(mess, msgType, self.request)

    def getAllWorkflowStates(self):
        """
        """
        plone_wf = getToolByName(self.context, 'portal_workflow')
        states = []
        for wf in plone_wf.getChildNodes():
            for s in wf.states.getChildNodes():
                if not(s.id in states):
                    states.append(s.id)
        return states

    def getAllWorkflowTransitions(self):
        """
        """
        plone_wf = getToolByName(self.context, 'portal_workflow')
        transitions = []
        for wf in plone_wf.getChildNodes():
            for t in wf.transitions.getChildNodes():
                if not(t.id in transitions):
                    transitions.append(t.id)
        return transitions



    def getServerImportableFiles(self):
        """
        """
        csvtool = getToolByName(self.context, "portal_csvreplicatatool")
        import_dir_path = csvtool.getServerfilepath()
        if import_dir_path is not None and import_dir_path!='':
            return os.listdir(import_dir_path)
        return []



# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2008 by ['Eric BREHAULT']
# Generator: ArchGenXML Version 2.0
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
__docformat__ = 'plaintext'


import logging
logger = logging.getLogger('csvreplicata: setuphandlers')
from Products.csvreplicata.config import PROJECTNAME
from Products.csvreplicata.config import DEPENDENCIES
import os
from Products.CMFCore.utils import getToolByName
import transaction
##code-section HEAD

import ConfigParser
from Products.csvreplicata.interfaces import ICSVReplicable, Icsvreplicata
from zope.interface import alsoProvides, noLongerProvides, directlyProvides

##/code-section HEAD

def isNotcsvreplicataProfile(context):
    return context.readDataFile("csvreplicata_marker.txt") is None

def setupHideToolsFromNavigation(context):
    """hide tools"""
    if isNotcsvreplicataProfile(context): return
    # uncatalog tools
    shortContext = context._profile_path.split(os.path.sep)[-3]
    if shortContext != 'csvreplicata': # avoid infinite recursions
        return
    site = context.getSite()
    toolnames = ['portal_csvreplicatatool']
    portalProperties = getToolByName(site, 'portal_properties')
    navtreeProperties = getattr(portalProperties, 'navtree_properties')
    if navtreeProperties.hasProperty('idsNotToList'):
        for toolname in toolnames:
            try:
                site[toolname].unindexObject()
            except:
                pass
            current = list(navtreeProperties.getProperty('idsNotToList') or [])
            if toolname not in current:
                current.append(toolname)
                kwargs = {'idsNotToList': current}
                navtreeProperties.manage_changeProperties(**kwargs)

def fixTools(context):
    """do post-processing on auto-installed tool instances"""
    if isNotcsvreplicataProfile(context): return
    site = context.getSite()

    tool_ids=['portal_csvreplicatatool']
    for tool_id in tool_ids:
        tool=site[tool_id]
        tool.initializeArchetype()



def updateRoleMappings(context):
    """after workflow changed update the roles mapping. this is like pressing
    the button 'Update Security Setting' and portal_workflow"""
    if isNotcsvreplicataProfile(context): return
    shortContext = context._profile_path.split(os.path.sep)[-3]
    if shortContext != 'csvreplicata': # avoid infinite recursions
        return
    wft = getToolByName(context.getSite(), 'portal_workflow')
    wft.updateRoleMappings()

def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotcsvreplicataProfile(context): return
    shortContext = context._profile_path.split(os.path.sep)[-3]
    if shortContext != 'csvreplicata': # avoid infinite recursions
        return
    site = context.getSite()




def importcsvStep(context):
    """This step give you a way to import content in your website with
    csvreplicata.

    How it works:
       read replicata.cfg file to load configuration

       read replicata.csv and call csvreplicata with the config.

    Config:
        [replicable_types]
        #List schemata for each types you want to import
        #Folder : default, categorization, dates, ownership, settings
        #News Item : default

        Document: default

        [settings]
        #list here every csv settings that will be given to replicator.importcsv
        encoding:utf-8
        delimiter:,
        stringdelimiter:"
        datetimeformat:%d/%m/%Y
        wf_transition = publish
        conflict_winner:SERVER

    """
    replicatacfg = context.openDataFile('replicata.cfg')
    replicatacsv = context.openDataFile('replicata.csv')
    if replicatacfg is None or replicatacsv is None:
        return
    site = context.getSite()
    wftool = getToolByName(site, 'portal_workflow')

    csvreplicatatool = getToolByName(site, 'portal_csvreplicatatool')
    # save existents user settings
    old_replicabletypes_settings = csvreplicatatool.replicabletypes

    #get import settings
    Config = ConfigParser.ConfigParser()
    Config.read(replicatacfg.name)
    options = Config.items('replicable_types', raw=True)
    replicable_types = {}
    for option in options:
        replicable_types[option[0].title()] = option[1].split(',')
    csvreplicatatool.replicabletypes = replicable_types

    #build kwargs of csvimport method.
    kwargs = {}

    wf_transition = Config.get('settings', 'wf_transition')
    if wf_transition:
        kwargs['wf_transition'] = wf_transition

    datetimeformat = Config.get('settings', 'datetimeformat')
    if datetimeformat:
        kwargs['datetimeformat'] = datetimeformat

    delimiter = Config.get('settings', 'delimiter')
    if delimiter:
        kwargs['delimiter'] = delimiter

    stringdelimiter = Config.get('settings', 'stringdelimiter')
    if stringdelimiter:
        kwargs['stringdelimiter'] = stringdelimiter

    encoding = Config.get('settings', 'encoding')
    if encoding:
        kwargs['encoding'] = encoding

    conflict_winner = Config.get('settings', 'conflict_winner')
    if conflict_winner:
        kwargs['conflict_winner'] = conflict_winner

    wf_transition = Config.get('settings', 'wf_transition')
    if wf_transition:
        kwargs['wf_transition'] = wf_transition

    #retrive context:
    global_path = Config.get('settings', 'global_path')
    if global_path:
        try:
            folder = site.restrictedTraverse(global_path)
        except KeyError:
            folders = global_path.split('/')
            current = site
            for folder in folders:
                if not folder:
                    continue
                try:
                    current.invokeFactory('Folder', folder)
                finally:
                    current = current.get(folder)
                    current.reindexObject()
            folder = current
    else:
        folder = site

    # now import
    alsoProvides(folder, ICSVReplicable)
    replicator = Icsvreplicata(folder)
    replicator.csvimport(replicatacsv, **kwargs)

    # restore replicabletypes user settings
    csvreplicatatool.replicabletypes = old_replicabletypes_settings

##code-section FOOT
##/code-section FOOT
